<?php

/*
 * Copyright (C) 2014, NoccyLabs
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

namespace NoccyLabs\Filesystem;

/**
 * Generic interface for classes encapsulating file- and directory related
 * functionality for a local or remote filesystem. The interface implementations
 * should always be considered remote in the implementation, for example for
 * deploying files on a server, this could be done over ssh2/sftp or copied to
 * the local web root.
 *
 * @author Christopher Vagnetoft <cvagnetoft@gmail.com>
 * @license GPL-3.0
 */
interface FilesystemInterface
{
    /**
     * Send a file to the remote filesystem from the local filesystem. This
     * could imply a local copy operation depending on the use.
     *
     * @param string The local file to send
     * @param string The remote file to write to
     * @param int The create mode (default 0644 octal)
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function send($local, $remote, $create_mode=0644);
    
    /**
     * Receive a file from the remote fileystem. This could imply a local copy
     * operation depending on the use.
     *
     * @param string The remote file to receive
     * @param string The local file to write 
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function receive($remote, $local);
    
    /**
     * Change the mode of a file.
     *
     * @param string The remote filename to modify
     * @param int The new numeric file mode 
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function chmod($remote, $mode);

    /**
     * Create a directory in the filesystem.
     *
     * @param string The name of the directory to create
     * @param int The mode to apply (default 0777 octal)
     * @param bool If true, parent directories will be created as needed.
     * @throws NoccyLabs\Filesystem\FilesystemException
     */    
    public function mkdir($remote, $mode=0777, $recursive=false);
    
    /**
     * Read the target of a link.
     *
     * @param string The link to read
     * @return string The link target
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function readlink($remote);
    
    /**
     * Return the real pathname of the file or directory.
     *
     * @param string The file to resolve the real path name for
     * @return string The real path name
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function realpath($remote);
    
    /**
     * Rename a file on the filesystem
     *
     * @param string The current filename
     * @param string The new filename
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function rename($from, $to);

    /**
     * Stat a file on the filesystem
     *
     * @return array File stat information
     * @throws NoccyLabs\Filesystem\FilesystemException
     */   
    public function stat($remote);
    
    /**
     * Create a symbolic link to a target
     *
     * @param string The target of the symbolic link
     * @param string The name of the link to create
     * @throw NoccyLabs\Filesystem\FilesystemException
     */
    public function symlink($target, $link);
    
    /**
     * Remove a file or directory.
     *
     * @param string The file or directory to remove.
     * @throws NoccyLabs\Filesystem\FilesystemException
     */
    public function unlink($remote);
}
